@@ -4,14 +4,15 @@
 #include "main.h"
 #include "global.h"
 #include "memory.h"
+#include "debug.h"
 
 #define PATH_STR_SIZE	16
 
 /*
  * Transforms the path group vector into a proper device map string
  */
-static void
-assemble_map (struct multipath * mp, char * str, vector pg)
+void
+assemble_map (struct multipath * mp)
 {
 	int i, j;
 	char * p;
@@ -24,11 +25,18 @@ assemble_map (struct multipath * mp, char * str, vector pg)
 
 	pp = VECTOR_SLOT(mp->paths, 0);
 
-	/* universal settings */
+	/*
+	 * select the right path selector :
+	 * 1) set internal default
+	 * 2) override by controler wide settings
+	 * 3) override by LUN specific settings
+	 */
+
+	/* 1) set internal default */
 	selector = conf->default_selector;
 	selector_args = conf->default_selector_args;
 
-	/* override by controler wide settings */
+	/* 2) override by controler wide settings */
 	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
 		hwe = VECTOR_SLOT(conf->hwtable, i);
 
@@ -41,7 +49,7 @@ assemble_map (struct multipath * mp, char * str, vector pg)
 		}
 	}
 
-	/* override by LUN specific settings */
+	/* 3) override by LUN specific settings */
 	for (i = 0; i < VECTOR_SIZE(conf->mptable); i++) {
 		mpe = VECTOR_SLOT(conf->mptable, i);
 
@@ -53,11 +61,11 @@ assemble_map (struct multipath * mp, char * str, vector pg)
 		}
 	}
 
-	p = str;
-	p += sprintf(p, " %i", VECTOR_SIZE(pg));
+	p = mp->params;
+	p += sprintf(p, " %i", VECTOR_SIZE(mp->pg));
 
-	for (i = 0; i < VECTOR_SIZE(pg); i++) {
-		pgpaths = VECTOR_SLOT(pg, i);
+	for (i = 0; i < VECTOR_SIZE(mp->pg); i++) {
+		pgpaths = VECTOR_SLOT(mp->pg, i);
 		p += sprintf(p, " %s %i %i",
 			     selector, VECTOR_SIZE(pgpaths), selector_args);
 
@@ -67,67 +75,20 @@ assemble_map (struct multipath * mp, char * str, vector pg)
 	}
 }
 
-/*
- * Two paths groups at most : one for paths with TUR==OK, one for the other
- * Needed for controlers that present ghost paths
- */
-extern void
-group_by_tur (struct multipath * mp, char * str) {
-	int i;
-	struct path * pp;
-	vector pg;
-	vector pgpaths_left;
-	vector pgpaths_right;
-	char * pathstr;
-
-	pgpaths_left = vector_alloc();
-	pgpaths_right = vector_alloc();
-
-	for (i = 0; i < VECTOR_SIZE(mp->paths); i++) {
-		pp = VECTOR_SLOT(mp->paths, i);
-		pathstr = zalloc(PATH_STR_SIZE);
-		strncpy(pathstr, pp->dev_t, strlen(pp->dev_t) - 1);
-
-		if (pp->tur) {
-			vector_alloc_slot(pgpaths_left);
-			vector_set_slot(pgpaths_left, pathstr);
-		} else {
-			vector_alloc_slot(pgpaths_right);
-			vector_set_slot(pgpaths_right, pathstr);
-		}
-	}
-	pg = vector_alloc();
-	vector_alloc_slot(pg);
-
-	if (!VECTOR_SIZE(pgpaths_left))
-		vector_set_slot(pg, pgpaths_right);
-
-	else if (!VECTOR_SIZE(pgpaths_right))
-		vector_set_slot(pg, pgpaths_left);
-
-	else {
-		vector_set_slot(pg, pgpaths_left);
-		vector_alloc_slot(pg);
-		vector_set_slot(pg, pgpaths_right);
-	}
-	assemble_map(mp, str, pg);
-}
-
 /*
  * One path group per unique serial number present in the path vector
  * Simple rotation logic for the head pg's serial
  */
 extern void
-group_by_serial (struct multipath * mp, int slot, char * str) {
+group_by_serial (struct multipath * mp, int slot) {
 	int i, k;
 	int * bitmap;
 	struct path * pp;
 	struct path * pp2;
-	vector pg;
 	vector pgpaths;
 	char * pathstr;
 	
-	pg = vector_alloc();
+	mp->pg = vector_alloc();
 
 	/* init the bitmap */
 	bitmap = zalloc(VECTOR_SIZE(mp->paths) * sizeof (int));
@@ -144,8 +105,8 @@ group_by_serial (struct multipath * mp, int slot, char * str) {
 
 		/* here, we really got a new pg */
 		pgpaths = vector_alloc();
-		vector_alloc_slot(pg);
-		vector_set_slot(pg, pgpaths);
+		vector_alloc_slot(mp->pg);
+		vector_set_slot(mp->pg, pgpaths);
 
 		/* feed the first path */
 		pathstr = zalloc(PATH_STR_SIZE);
@@ -184,8 +145,8 @@ even:
 
 		/* here, we really got a new pg */
 		pgpaths = vector_alloc();
-		vector_alloc_slot(pg);
-		vector_set_slot(pg, pgpaths);
+		vector_alloc_slot(mp->pg);
+		vector_set_slot(mp->pg, pgpaths);
 
 		/* feed the first path */
 		pathstr = zalloc(PATH_STR_SIZE);
@@ -214,56 +175,56 @@ even:
 		}
 	}
 	free(bitmap);
-	assemble_map(mp, str, pg);
 }
 
-/*
- * This is for pure failover
- */
 extern void
-one_path_per_group (struct multipath * mp, char * str)
+one_path_per_group (struct multipath * mp)
 {
 	int i;
 	char * pathstr;
 	struct path * pp;
-	vector pg;
 	vector pgpaths;
 
-	pg = vector_alloc();
+	mp->pg = vector_alloc();
 	
-	for (i=0; i < VECTOR_SIZE(mp->paths); i++) {
+	for (i = 0; i < VECTOR_SIZE(mp->paths); i++) {
 		pp = VECTOR_SLOT(mp->paths, i);
 
 		if (0 != pp->sg_id.scsi_type)
 			continue;
 
-		pathstr = zalloc(PATH_STR_SIZE);
-		strncpy(pathstr, pp->dev_t, strlen(pp->dev_t) - 1);
-		pgpaths = vector_alloc();
-		vector_alloc_slot(pgpaths);
-		vector_set_slot(pgpaths, pathstr);
-		vector_alloc_slot(pg);
-		vector_set_slot(pg, pgpaths);
+		if (!pp->tur) {
+			pathstr = zalloc(PATH_STR_SIZE);
+			strncpy(pathstr, pp->dev_t, strlen(pp->dev_t) - 1);
+			pgpaths = vector_alloc();
+			vector_alloc_slot(pgpaths);
+			vector_set_slot(pgpaths, pathstr);
+			vector_alloc_slot(mp->pg);
+			vector_set_slot(mp->pg, pgpaths);
+		} else {
+			pathstr = zalloc(PATH_STR_SIZE);
+			strncpy(pathstr, pp->dev_t, strlen(pp->dev_t) - 1);
+			pgpaths = vector_alloc();
+			vector_alloc_slot(pgpaths);
+			vector_set_slot(pgpaths, pathstr);
+			vector_alloc_slot(mp->pg);
+			vector_set_slot(mp->pg, pgpaths);
+		}
 	}
-	assemble_map(mp, str, pg);
 }
 
-/*
- * This is for symmetric array controlers, with no switch penalty
- */
 extern void
-one_group (struct multipath * mp, char * str)
+one_group (struct multipath * mp)
 {
 	int i;
 	char * pathstr;
 	struct path * pp;
-	vector pg;
-	vector pgpaths;
+	vector pgvalidpaths;
+	vector pgfailedpaths;
 
-	pgpaths = vector_alloc();
-	pg = vector_alloc();
-	vector_alloc_slot(pg);
-	vector_set_slot(pg, pgpaths);
+	pgvalidpaths = vector_alloc();
+	pgfailedpaths = vector_alloc();
+	mp->pg = vector_alloc();
 
 	for (i = 0; i < VECTOR_SIZE(mp->paths); i++) {
 		pp = VECTOR_SLOT(mp->paths, i);
@@ -271,10 +232,24 @@ one_group (struct multipath * mp, char * str)
 		if (0 != pp->sg_id.scsi_type)
 			continue;
 
-		pathstr = zalloc(PATH_STR_SIZE);
-		strncpy(pathstr, pp->dev_t, strlen(pp->dev_t) - 1);
-		vector_alloc_slot(pgpaths);
-		vector_set_slot(pgpaths, pathstr);
+		if (!pp->tur) {
+			pathstr = zalloc(PATH_STR_SIZE);
+			strncpy(pathstr, pp->dev_t, strlen(pp->dev_t) - 1);
+			vector_alloc_slot(pgfailedpaths);
+			vector_set_slot(pgfailedpaths, pathstr);
+		} else {
+			pathstr = zalloc(PATH_STR_SIZE);
+			strncpy(pathstr, pp->dev_t, strlen(pp->dev_t) - 1);
+			vector_alloc_slot(pgvalidpaths);
+			vector_set_slot(pgvalidpaths, pathstr);
+		}
+	}
+	if (VECTOR_SIZE(pgvalidpaths) > 0) {
+		vector_alloc_slot(mp->pg);
+		vector_set_slot(mp->pg, pgvalidpaths);
+	}
+	if (VECTOR_SIZE(pgfailedpaths) > 0) {
+		vector_alloc_slot(mp->pg);
+		vector_set_slot(mp->pg, pgfailedpaths);
 	}
-	assemble_map(mp, str, pg);
 }
